@@ -80,6 +80,12 @@ def mag_heading(RAW_IMU, ATTITUDE, declination=None, SENSOR_OFFSETS=None, ofs=No
         heading += 360
     return heading
 
+def gps_time_to_epoch(week, msec):
+    '''convert GPS week and TOW to a time in seconds since 1970'''
+    epoch = 86400*(10*365 + int((1980-1969)/4) + 1 + 6 - 2)
+    return epoch + 86400*7*week + msec*0.001 - 18
+
+
 def mag_heading_motors(RAW_IMU, ATTITUDE, declination, SENSOR_OFFSETS, ofs, SERVO_OUTPUT_RAW, motor_ofs):
     '''calculate heading from raw magnetometer'''
     ofs = get_motor_offsets(SERVO_OUTPUT_RAW, ofs, motor_ofs)
@@ -231,6 +237,24 @@ def lowpass(var, key, factor):
         lowpass_data[key] = factor*lowpass_data[key] + (1.0 - factor)*var
     return lowpass_data[key]
 
+def lpalpha(sample_rate_hz, cutoff_hz):
+    '''find alpha for low pass filter'''
+    rc = 1.0 / (2*pi*cutoff_hz)
+    dt = 1.0 / sample_rate_hz
+    return 1.0 - dt/(dt+rc)
+
+lowpass_hz_data = {}
+
+def lowpassHz(var, key, sample_rate_hz, cutoff_hz):
+    '''a simple lowpass filter with specified frequency'''
+    global lowpass_hz_data
+    alpha = lpalpha(sample_rate_hz, cutoff_hz)
+    if not key in lowpass_hz_data:
+        lowpass_hz_data[key] = var
+    else:
+        lowpass_hz_data[key] = alpha*lowpass_hz_data[key] + (1.0-alpha)*var
+    return lowpass_hz_data[key]
+
 last_diff = {}
 
 def diff(var, key):
@@ -266,6 +290,33 @@ def delta(var, key, tusec=None):
     last_delta[key] = (var, tnow, ret)
     return ret
 
+last_sum = {}
+
+def sum(var, key):
+    '''sum variable'''
+    global last_sum
+    ret = 0
+    if not key in last_sum:
+        last_sum[key] = 0
+    last_sum[key] += var
+    return last_sum[key]
+
+last_integral = {}
+
+def integral(var, key, timeus):
+    '''integrate variable'''
+    global last_integral
+    ret = 0
+    if not key in last_integral:
+        last_integral[key] = (0,timeus)
+    (lastsum,lastt) = last_integral[key]
+    dt = (timeus - lastt) * 1.0e-6
+    dv = var * dt
+    newv = lastsum + dv
+    last_integral[key] = (newv,timeus)
+    return newv
+
+
 def delta_angle(var, key, tusec=None):
     '''calculate slope of an angle'''
     global last_delta
@@ -1369,10 +1420,14 @@ def reset_state_data():
     global first_fix
     global dcm_state
     global earth_field
+    global last_sum
+    global last_integral
     average_data.clear()
     derivative_data.clear()
     lowpass_data.clear()
     last_delta.clear()
+    last_sum.clear()
+    last_integral.clear()
     first_fix = None
     dcm_state = None
     earth_field = None
