@@ -32,10 +32,12 @@ parser.add_argument("--condition", default=None, help="select packets by conditi
 parser.add_argument("-q", "--quiet", action='store_true', help="don't display packets")
 parser.add_argument("-o", "--output", default=None, help="output matching packets to give file")
 parser.add_argument("-p", "--parms", action='store_true', help="preserve parameters in output with -o")
-parser.add_argument("--format", default=None, help="Change the output format between 'standard', 'json', and 'csv'. For the CSV output, you must supply types that you want.")
+parser.add_argument("--format", default=None, help="Change the output format between 'standard', 'json', 'csv' and 'mat'. For the CSV output, you must supply types that you want. For MAT output, specify output file with --mat_file")
 parser.add_argument("--csv_sep", dest="csv_sep", default=",", help="Select the delimiter between columns for the output CSV file. Use 'tab' to specify tabs. Only applies when --format=csv")
 parser.add_argument("--types", default=None, help="types of messages (comma separated with wildcard)")
 parser.add_argument("--nottypes", default=None, help="types of messages not to include (comma separated with wildcard)")
+parser.add_argument("--mat_file", dest="mat_file", help="Output file path for MATLAB file output. Only applies when --format=mat")
+parser.add_argument("-c", "--compress", action='store_true', help="Compress .mat file data")
 parser.add_argument("--dialect", default="ardupilotmega", help="MAVLink dialect")
 parser.add_argument("--zero-time-base", action='store_true', help="use Z time base for DF logs")
 parser.add_argument("--no-bad-data", action='store_true', help="Don't output corrupted messages")
@@ -50,6 +52,7 @@ parser.add_argument("--mav10", action='store_true', help="parse as MAVLink1")
 parser.add_argument("--reduce", type=int, default=0, help="reduce streaming messages")
 parser.add_argument("log", metavar="LOG")
 parser.add_argument("--profile", action='store_true', help="run the Yappi python profiler")
+parser.add_argument("--meta", action='store_true', help="output meta-data msgs even if not matching condition")
 
 args = parser.parse_args()
 
@@ -65,6 +68,11 @@ if args.profile:
     import yappi    # We do the import here so that we won't barf if run normally and yappi not available
     yappi.start()
 
+if args.format == 'mat':
+    # Load these modules here, as they're only needed for MAT file creation
+    import scipy.io
+    import numpy as np
+
 filename = args.log
 mlog = mavutil.mavlink_connection(filename, planner_format=args.planner,
                                   notimestamps=args.notimestamps,
@@ -174,6 +182,7 @@ if isbin and args.format == 'csv':
     match_types.append("FMT")
 
 # Keep track of data from the current timestep. If the following timestep has the same data, it's stored in here as well. Output should therefore have entirely unique timesteps.
+MAT = {}    # Dictionary to hold output data for 'mat' format option
 while True:
     m = mlog.recv_match(blocking=args.follow, type=match_types)
     if m is None:
@@ -204,7 +213,8 @@ while True:
             output.write(struct.pack('>Q', int(timestamp*1.0e6)) + m.get_msgbuf())
             continue
 
-    if not mavutil.evaluate_condition(args.condition, mlog.messages):
+    if not mavutil.evaluate_condition(args.condition, mlog.messages) and (
+            not (m.get_type() in ['FMT', 'FMTU', 'MULT','PARM'] and args.meta)):
         continue
     if args.source_system is not None and args.source_system != m.get_srcSystem():
         continue
@@ -292,6 +302,29 @@ while True:
                 csv_out = [str(data[y]) if y != "timestamp" else "" for y in fields]
             else:
                 csv_out = [str(data[y.split('.')[-1]]) if y.split('.')[0] == type and y.split('.')[-1] in data else "" for y in fields]
+    # MAT format outputs data to a .mat file specified through the
+    # --mat_file option
+    elif args.format == 'mat':
+        # If this packet contains data (i.e. is not a FMT
+        # packet), append the data in this packet to the
+        # corresponding list
+        if m.get_type()!='FMT':
+
+            # If this packet type has not yet been
+            # seen, add a new entry to the big dict
+            if m.get_type() not in MAT:
+                MAT[m.get_type()] = {}
+
+            md = m.to_dict()
+            del md['mavpackettype']
+            cols = md.keys()
+            for col in cols:
+                # If this column hasn't had data entered,
+                # make a new key and list
+                if col in MAT[m.get_type()]:
+                    MAT[m.get_type()][col].append(md[col])
+                else:
+                    MAT[m.get_type()][col] = [md[col]]
     elif args.show_types:
         # do nothing
         pass
@@ -312,6 +345,10 @@ while True:
     # Update our last timestamp value.
     last_timestamp = timestamp
 
+# Export the .mat file
+if args.format == 'mat':
+    scipy.io.savemat(args.mat_file, MAT, do_compression=args.compress)
+
 if args.show_types:
     for msgType in available_types:
         print(msgType)
